import java.awt.geom.*;
import java.awt.*;
import java.util.*;

/**
 * Representacion grafica de una fuente ideal de voltaje continuo.
 * @author Carlos Cortes
 */
public class FigFuenteCC extends Figura
{
    /**
     */
    public FigFuenteCC()
    {
        //Dibujo de la fuente
        figList = new ArrayList < Shape > ();
        figList.add(new Line2D.Float(-8,  -6, 8, -6));
        figList.add(new Line2D.Float(-5,  -2, 5, -2));
        figList.add(new Line2D.Float(-8,   2, 8,  2));
        figList.add(new Line2D.Float(-5,   6, 5,  6));
        figList.add(new Line2D.Float(4, -11, 8, -11));
        figList.add(new Line2D.Float(6, -14, 6, -8));
        //Dibujo de los pines
        figList.add(new Line2D.Float(0, -32, 0, -6));
        figList.add(new Line2D.Float(0,  32, 0,  6));
        
        //ubicacion de los pines (para conectarlos)
        pinList = new ArrayList < Point > ();
        pinList.add(new Point(0, -32));
        pinList.add(new Point(0, 32));
        
        angulo = 0;
        centro = new Point(0, 0);
        zona = new Rectangle(-8, -11, 17, 23);
        posTexto = new Point(10, 4);
        this.trasladar(Defs.DEFAULT_POSICION_X, Defs.DEFAULT_POSICION_Y);
        
        /* tramite para cumplir con sjsu, que no me permite definir estos
           atributos en Figura.java como 'protected' */
        super.setCentro(centro);
        super.setPosTexto(posTexto);
        super.setFigList(figList);
        super.setBounds(zona);
        super.setPinList(pinList);
    }
    
    /**
     * Rota la forma respecto a su punto central en 90 grados.
     * (en sentido reloj)
     */
    public void rotar()
    {
        int i, x, y, w, h;
        Line2D.Float linea;
        /* Rotacion de los elementos */
        for (i = 0; i < figList.size(); i++)
        {
            linea = (Line2D.Float)figList.get(i);
            float x1 = (float)(linea.getY1() - centro.getY() - centro.getX());
            float y1 = (float)(linea.getX1() - centro.getX() + centro.getY());
            float x2 = (float)(linea.getY2() - centro.getY() - centro.getX());
            float y2 = (float)(linea.getX2() - centro.getX() + centro.getY());
            linea.setLine(-x1, y1, -x2, y2);
        }
        /* Rotacion de los pines */
        for (i = 0; i < pinList.size(); i++)
        {
            x = (int)(pinList.get(i).getY() - centro.getY() - centro.getX());
            y = (int)(pinList.get(i).getX() - centro.getX() + centro.getY());
            pinList.get(i).setLocation(-x, y);
        }
        /* Rotacion de 'zona' */
        x = (int)(zona.getY() - centro.getY() - centro.getX());
        y = (int)(zona.getX() - centro.getX() + centro.getY());
        w = (int)(zona.getWidth());
        h = (int)(zona.getHeight());
        zona.setBounds(-x - h + 1, y, h, w);
        /* Rotacion del texto */
        angulo = (angulo + 1) % 4;
        if (angulo % 2 == 0)
        {
            x = 10 + (int)centro.getX();
            y =  4 + (int)centro.getY();
        }
        else
        {
            x = -8 + (int)centro.getX();
            y = -10 + (int)centro.getY();
        }
        posTexto.setLocation(x, y);
    }
    
    /**
     * Traslada el componente al punto (newX, newY).
     * @param newX nueva coordenada 'x'.
     * @param newY nueva coordenada 'y'.
     */
    public void trasladar(int newX, int newY)
    {
        int i, x, y;
        Line2D.Float linea;
        
        /* Trasladando las lineas */
        for (i = 0; i < figList.size(); i++)
        {
            linea = (Line2D.Float)figList.get(i);
            float x1 = (float)(linea.getX1() - centro.getX()) + newX;
            float y1 = (float)(linea.getY1() - centro.getY()) + newY;
            float x2 = (float)(linea.getX2() - centro.getX()) + newX;
            float y2 = (float)(linea.getY2() - centro.getY()) + newY;
            linea.setLine(x1, y1, x2, y2);
        }
        
        /* Trasladando los pines */
        for (i = 0; i < pinList.size(); i++)
        {
            x = (int)(pinList.get(i).getX() - centro.getX()) + newX;
            y = (int)(pinList.get(i).getY() - centro.getY()) + newY;
            pinList.get(i).setLocation(x, y);
        }
        
        /* Trasladando 'zona' */
        x = (int)(zona.getX() - centro.getX()) + newX;
        y = (int)(zona.getY() - centro.getY()) + newY;
        zona.setLocation(x, y);
        
        /* Trasladando Texto */
        x = (int)(posTexto.getX() - centro.getX()) + newX;
        y = (int)(posTexto.getY() - centro.getY()) + newY;
        posTexto.setLocation(x, y);
        
        /* Trasladando 'centro' */
        centro.setLocation(newX, newY);
    }
    
    /**
     * Punto central de la figura
     */
    private Point centro;
    
    /**
     * Angulo de la figura. 0 es 0 grados, 1 es 90 grados,
     * 2 es 180 grados y 3 es 270 grados.
     * (en sentido reloj)
     */
    private int angulo;
    
    /**
     * Posicion de la esquina inferior-izquierda del cuadro
     * de texto que muestra el valor del componente.
     */
    private Point posTexto;
    
    /**
     * Lista de las formas basicas (Shapes) que componen la figura.
     */
    private ArrayList < Shape > figList;
    
    /**
     * Zona activa. La que detecta el mouse.
     */
    private Rectangle zona;
    
    /**
     * Lista que almacena la posicion grafica de los pines de la figura.
     */
    private ArrayList < Point > pinList;
}
