import java.awt.*;
import java.util.*;

/**
 * Representacion grafica de un componente electronico.
 * @author Carlos Cortes
 */
public abstract class Figura
{
    /**
     * @return La posicion del punto central de la figura que corresponde
     * al componente.
     */
    public Point getCentro()
    {
        return centro;
    }
    
    /**
     * @return La lista de los pines de la figura que corresponde
     * al componente.
     */
    public ArrayList < Point > getPinList()
    {
        return pinList;
    }
    
    /**
     * @return La posicion del cuadro de texto de la figura
     * que corresponde al componente.
     */
    public Point getPosTexto()
    {
        return posTexto;
    }
    
    /**
     * @return La cantidad de <Shapes> que componen la figura.
     */
    public ArrayList < Shape > getFigList()
    {
        return figList;
    }
    
    /**
     * @return La cantidad de elementos (Shapes) que componen la figura,
     * que es la representacion grafica del componente.
     */
    public int numShapes()
    {
        return figList.size();
    }
    
    /**
     * @param n Indice del elemento (Shape) que se quiere
     * obtener de la lista figList.
     * @return El elemento (Shape) 'n', de los que componen la figura
     * que corresponde al componente.
     */
    public Shape getShape(int n)
    {
        return figList.get(n);
    }
    
    /**
     * @return Un rectangulo que define el area activa de la figura,
     * por donde debe pasar el mouse para seleccionarla.
     */
    public Rectangle getBounds()
    {
        return zona;
    }
    
    /**
     * @param c Posicion del centro de la figura.
     */
    public void setCentro(Point c)
    {
        centro = c;
    }
    
    /**
     * @param pt Posicion de la esquina inferior-izquierda del cuadro
     * de texto que muestra el valor del componente.
     */
    public void setPosTexto(Point pt)
    {
        posTexto = pt;
    }
    
    /**
     * @param fl Lista de las formas basicas (Shapes) que componen la figura.
     */
    public void setFigList(ArrayList < Shape > fl)
    {
        figList = fl;
    }
    
    /**
     * @param z Zona activa. La que detecta el mouse.
     */
    public void setBounds(Rectangle z)
    {
        zona = z;
    }
    
    /**
     * @param pl Lista que almacena la posicion grafica de los pines de
     * la figura.
     */
    public void setPinList(ArrayList < Point > pl)
    {
        pinList = pl;
    }
    
    /**
     * Rota la forma respecto a su punto central en 90 grados.
     * (en sentido reloj)
     */
    public abstract void rotar();

    /**
     * Traslada el componente al punto (x,y).
     * @param x nueva coordenada 'x'.
     * @param y nueva coordenada 'y'.
     */
    public abstract void trasladar(int x, int y);
    
    /**
     * Punto central de la figura
     */
    private Point centro;
    
    /**
     * Posicion de la esquina inferior-izquierda del cuadro
     * de texto que muestra el valor del componente.
     */
    private Point posTexto;
    
    /**
     * Lista de las formas basicas (Shapes) que componen la figura.
     */
    private ArrayList < Shape > figList;
    
    /**
     * Zona activa. La que detecta el mouse.
     */
    private Rectangle zona;
    
    /**
     * Lista que almacena la posicion grafica de los pines de la figura.
     */
    private ArrayList < Point > pinList;
}
